@@ -35,11 +35,10 @@ print "    Square = (" . $s->x . "," . $s->y . ")\n";
 
 # ----- Call some methods -----
 
-print "\nHere are some properties of the shapes:\n";
+print "\nCall some overloaded methods:\n";
 foreach (array(1, 2.1, "quick brown fox", $c, $s) as $o) {
-      print "    ".get_class($o)." \$o\n";
-      print "        overloaded = " .  overloaded($o) . "\n";
-  }
+  print "        overloaded = " .  overloaded($o) . "\n";
+}
 
 # Need to unset($o) or else we hang on to a reference to the Square object.
 unset($o);
