@@ -10,3 +10,11 @@ assert issubclass(IntSet, MutableSet)
 assert issubclass(IntMultiset, MutableSet)
 assert issubclass(IntVector, MutableSequence)
 assert issubclass(IntList, MutableSequence)
+
+mapii = Mapii()
+multimapii = Multimapii()
+intset = IntSet()
+intmultiset = IntMultiset()
+intvector = IntVector()
+intlist = IntList()
+
