@@ -12,8 +12,6 @@
  *     DOH objects.  A number of small utility functions are also included.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_base_c[] = "$Id: base.c 12167 2010-07-22 16:59:29Z vadz $";
-
 #include "dohint.h"
 
 /* -----------------------------------------------------------------------------
@@ -259,7 +257,7 @@ int DohEqual(const DOH *obj1, const DOH *obj2) {
 
     if (!b1info) {
       return obj1 == obj2;
-    } else if ((b1info == b2info)) {
+    } else if (b1info == b2info) {
       return b1info->doh_equal ? (b1info->doh_equal) (b1, b2) : (b1info->doh_cmp ? (b1info->doh_cmp) (b1, b2) == 0 : (b1 == b2));
     } else {
       return 0;
@@ -761,6 +759,7 @@ int DohUngetc(int ch, DOH *obj) {
  * DohClose()
  * ----------------------------------------------------------------------------- */
 
+/*
 int DohClose(DOH *obj) {
   DohBase *b = (DohBase *) obj;
   DohObjInfo *objinfo;
@@ -773,6 +772,7 @@ int DohClose(DOH *obj) {
   }
   return fclose((FILE *) obj);
 }
+*/
 
 /* -----------------------------------------------------------------------------
  * DohIsString()
