@@ -12,8 +12,6 @@
  *     file semantics.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_string_c[] = "$Id: string.c 12536 2011-03-14 07:22:08Z wsfulton $";
-
 #include "dohint.h"
 
 extern DohObjInfo DohStringType;
@@ -651,6 +649,21 @@ static char *match_identifier_end(char *base, char *s, char *token, int tokenlen
   return 0;
 }
 
+static char *match_number_end(char *base, char *s, char *token, int tokenlen) {
+  (void) base;
+  while (s) {
+    s = strstr(s, token);
+    if (!s)
+      return 0;
+    if (isdigit((int) *(s + tokenlen))) {
+      s += tokenlen;
+      continue;
+    }
+    return s;
+  }
+  return 0;
+}
+
 /* -----------------------------------------------------------------------------
  * replace_simple()
  *
@@ -899,6 +912,8 @@ static int String_replace(DOH *stro, const DOHString_or_char *token, const DOHSt
     return replace_simple(str, Char(token), Char(rep), flags, count, match_identifier_begin);
   } else if (flags & DOH_REPLACE_ID) {
     return replace_simple(str, Char(token), Char(rep), flags, count, match_identifier);
+  } else if (flags & DOH_REPLACE_NUMBER_END) {
+    return replace_simple(str, Char(token), Char(rep), flags, count, match_number_end);
   } else {
     return replace_simple(str, Char(token), Char(rep), flags, count, match_simple);
   }
@@ -1136,11 +1151,7 @@ DOHString *DohNewStringf(const DOHString_or_char *fmt, ...) {
 int DohStrcmp(const DOHString_or_char *s1, const DOHString_or_char *s2) {
   const char *c1 = Char(s1);
   const char *c2 = Char(s2);
-  if (c1 && c2) {
-    return strcmp(c1, c2);
-  } else {
-    return c1 < c2;
-  }
+  return strcmp(c1, c2);
 }
 
 int DohStrncmp(const DOHString_or_char *s1, const DOHString_or_char *s2, int n) {
