@@ -89,6 +89,7 @@
 #define WARN_PARSE_REC_INHERITANCE    323
 #define WARN_PARSE_NESTED_TEMPLATE    324
 #define WARN_PARSE_NAMED_NESTED_CLASS 325
+#define WARN_PARSE_EXTEND_NAME        326
 
 #define WARN_IGNORE_OPERATOR_NEW        350	/* new */
 #define WARN_IGNORE_OPERATOR_DELETE     351	/* delete */
@@ -195,6 +196,9 @@
 #define WARN_LANG_PORTABILITY_FILENAME 518
 #define WARN_LANG_TEMPLATE_METHOD_IGNORE 519
 #define WARN_LANG_SMARTPTR_MISSING    520
+#define WARN_LANG_ILLEGAL_DESTRUCTOR  521
+#define WARN_LANG_EXTEND_CONSTRUCTOR  522
+#define WARN_LANG_EXTEND_DESTRUCTOR   523
 
 /* -- Reserved (600-799) -- */
 
@@ -240,6 +244,7 @@
 #define WARN_JAVA_TYPEMAP_JAVACONSTRUCT_UNDEF 823
 #define WARN_JAVA_TYPEMAP_DIRECTORIN_NODESC   824
 #define WARN_JAVA_NO_DIRECTORCONNECT_ATTR     825
+#define WARN_JAVA_NSPACE_WITHOUT_PACKAGE      826
 
 /* please leave 810-829 free for Java */
 
