@@ -15,8 +15,6 @@
  * Doc/Manual/SWIGPlus.html for details.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_allocate_cxx[] = "$Id: allocate.cxx 12804 2011-09-13 06:15:29Z wsfulton $";
-
 #include "swigmod.h"
 #include "cparse.h"
 
@@ -43,7 +41,7 @@ extern "C" {
 	  SwigType *decl1 = SwigType_typedef_resolve_all(decl);
 	  SwigType *decl2 = SwigType_pop_function(decl1);
 	  if (Strcmp(decl2, search_decl) == 0) {
-	    if (!Getattr(n, "abstract")) {
+	    if (!GetFlag(n, "abstract")) {
 	      Delete(decl1);
 	      Delete(decl2);
 	      return 1;
@@ -327,38 +325,36 @@ class Allocate:public Dispatcher {
       Swig_symbol_setscope(oldtab);
       return ret;
     }
-    List *abstract = Getattr(base, "abstract");
-    if (abstract) {
+    List *abstracts = Getattr(base, "abstracts");
+    if (abstracts) {
       int dabstract = 0;
-      int len = Len(abstract);
+      int len = Len(abstracts);
       for (int i = 0; i < len; i++) {
-	Node *nn = Getitem(abstract, i);
+	Node *nn = Getitem(abstracts, i);
 	String *name = Getattr(nn, "name");
 	if (!name)
 	  continue;
+	if (Strchr(name, '~'))
+	  continue;		/* Don't care about destructors */
 	String *base_decl = Getattr(nn, "decl");
 	if (base_decl)
 	  base_decl = SwigType_typedef_resolve_all(base_decl);
-	if (Strchr(name, '~'))
-	  continue;		/* Don't care about destructors */
-
-	if (SwigType_isfunction(base_decl)) {
+	if (SwigType_isfunction(base_decl))
 	  search_decl = SwigType_pop_function(base_decl);
-	}
 	Node *dn = Swig_symbol_clookup_local_check(name, 0, check_implemented);
 	Delete(search_decl);
 	Delete(base_decl);
 
 	if (!dn) {
-	  List *nabstract = Getattr(n, "abstract");
-	  if (!nabstract) {
-	    nabstract = NewList();
-	    Setattr(n, "abstract", nabstract);
-	    Delete(nabstract);
+	  List *nabstracts = Getattr(n, "abstracts");
+	  if (!nabstracts) {
+	    nabstracts = NewList();
+	    Setattr(n, "abstracts", nabstracts);
+	    Delete(nabstracts);
 	  }
-	  Append(nabstract, nn);
-	  if (!Getattr(n, "abstract:firstnode")) {
-	    Setattr(n, "abstract:firstnode", nn);
+	  Append(nabstracts, nn);
+	  if (!Getattr(n, "abstracts:firstnode")) {
+	    Setattr(n, "abstracts:firstnode", nn);
 	  }
 	  dabstract = base != n;
 	}
@@ -415,7 +411,7 @@ class Allocate:public Dispatcher {
 		  match = 1;
 		  break;
 		}
-		if ((!symname || (!Getattr(e, "sym:name"))) && (Cmp(name, Getattr(e, "name")) == 0)) {
+		if (!Getattr(e, "sym:name") && (Cmp(name, Getattr(e, "name")) == 0)) {
 		  match = 1;
 		  break;
 		}
@@ -598,19 +594,19 @@ Allocate():
     /* Check if the class is abstract via inheritance.   This might occur if a class didn't have
        any pure virtual methods of its own, but it didn't implement all of the pure methods in
        a base class */
-    if (!Getattr(n, "abstract") && is_abstract_inherit(n)) {
+    if (!Getattr(n, "abstracts") && is_abstract_inherit(n)) {
       if (((Getattr(n, "allocate:public_constructor") || (!GetFlag(n, "feature:nodefault") && !Getattr(n, "allocate:has_constructor"))))) {
 	if (!GetFlag(n, "feature:notabstract")) {
-	  Node *na = Getattr(n, "abstract:firstnode");
+	  Node *na = Getattr(n, "abstracts:firstnode");
 	  if (na) {
 	    Swig_warning(WARN_TYPE_ABSTRACT, Getfile(n), Getline(n),
 			 "Class '%s' might be abstract, " "no constructors generated,\n", SwigType_namestr(Getattr(n, "name")));
 	    Swig_warning(WARN_TYPE_ABSTRACT, Getfile(na), Getline(na), "Method %s might not be implemented.\n", Swig_name_decl(na));
-	    if (!Getattr(n, "abstract")) {
-	      List *abstract = NewList();
-	      Append(abstract, na);
-	      Setattr(n, "abstract", abstract);
-	      Delete(abstract);
+	    if (!Getattr(n, "abstracts")) {
+	      List *abstracts = NewList();
+	      Append(abstracts, na);
+	      Setattr(n, "abstracts", abstracts);
+	      Delete(abstracts);
 	    }
 	  }
 	}
@@ -620,7 +616,7 @@ Allocate():
     if (!Getattr(n, "allocate:has_constructor")) {
       /* No constructor is defined.  We need to check a few things */
       /* If class is abstract.  No default constructor. Sorry */
-      if (Getattr(n, "abstract")) {
+      if (Getattr(n, "abstracts")) {
 	Delattr(n, "allocate:default_constructor");
       }
       if (!Getattr(n, "allocate:default_constructor")) {
@@ -641,7 +637,7 @@ Allocate():
       }
     }
     if (!Getattr(n, "allocate:has_copy_constructor")) {
-      if (Getattr(n, "abstract")) {
+      if (Getattr(n, "abstracts")) {
 	Delattr(n, "allocate:copy_constructor");
       }
       if (!Getattr(n, "allocate:copy_constructor")) {
