@@ -11,8 +11,6 @@
  * R language module for SWIG.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_r_cxx[] = "$Id: r.cxx 12536 2011-03-14 07:22:08Z wsfulton $";
-
 #include "swigmod.h"
 
 static const double DEFAULT_NUMBER = .0000123456712312312323;
@@ -39,10 +37,10 @@ static String * getRTypeName(SwigType *t, int *outCount = NULL) {
   if(Strncmp(b, "struct ", 7) == 0) 
     Replace(b, "struct ", "", DOH_REPLACE_FIRST);
   
-  /* Printf(stderr, "<getRTypeName> %s,base = %s\n", t, b);
+  /* Printf(stdout, "<getRTypeName> %s,base = %s\n", t, b);
      for(i = 0; i < Len(els); i++) 
-     Printf(stderr, "%d) %s, ", i, Getitem(els,i));
-     Printf(stderr, "\n"); */
+     Printf(stdout, "%d) %s, ", i, Getitem(els,i));
+     Printf(stdout, "\n"); */
   
   for(i = 0; i < Len(els); i++) {
     String *el = Getitem(els, i);
@@ -92,7 +90,7 @@ static String *getRClassName(String *retType, int /*addRef*/ = 1, int upRef=0) {
   if(!l || n == 0) {
 #ifdef R_SWIG_VERBOSE
     if (debugMode)
-      Printf(stderr, "SwigType_split return an empty list for %s\n", 
+      Printf(stdout, "SwigType_split return an empty list for %s\n", 
 	     retType);
 #endif
     return(tmp);
@@ -148,7 +146,7 @@ static String * getRClassNameCopyStruct(String *retType, int addRef) {
   int n = Len(l);
   if(!l || n == 0) {
 #ifdef R_SWIG_VERBOSE
-    Printf(stderr, "SwigType_split return an empty list for %s\n", retType);
+    Printf(stdout, "SwigType_split return an empty list for %s\n", retType);
 #endif
     return(tmp);
   }
@@ -292,10 +290,12 @@ public:
   int membervariableHandler(Node *n);
 
   int typedefHandler(Node *n);
+  static List *Swig_overload_rank(Node *n,
+			   bool script_lang_wrapping);
 
   int memberfunctionHandler(Node *n) {
     if (debugMode)
-      Printf(stderr, "<memberfunctionHandler> %s %s\n", 
+      Printf(stdout, "<memberfunctionHandler> %s %s\n", 
 	     Getattr(n, "name"),
 	     Getattr(n, "type"));
     member_name = Getattr(n, "sym:name");
@@ -497,11 +497,11 @@ int R::getFunctionPointerNumArgs(Node *n, SwigType *tt) {
   (void) tt;
   n = Getattr(n, "type");
   if (debugMode)
-    Printf(stderr, "type: %s\n", n);
+    Printf(stdout, "type: %s\n", n);
 
   ParmList *parms = Getattr(n, "parms");
   if (debugMode)
-    Printf(stderr, "parms = %p\n", parms);
+    Printf(stdout, "parms = %p\n", parms);
   return ParmList_len(parms);
 }
 
@@ -512,7 +512,7 @@ void R::addSMethodInfo(String *name, String *argType, int nargs) {
   if(!SMethodInfo)
     SMethodInfo = NewHash();
   if (debugMode)
-    Printf(stderr, "[addMethodInfo] %s\n", name);
+    Printf(stdout, "[addMethodInfo] %s\n", name);
 
   Hash *tb = Getattr(SMethodInfo, name);
 
@@ -543,7 +543,7 @@ String * R::createFunctionPointerHandler(SwigType *t, Node *n, int *numArgs) {
     return funName;
   
   if (debugMode)
-    Printf(stderr, "<createFunctionPointerHandler> Defining %s\n",  t);
+    Printf(stdout, "<createFunctionPointerHandler> Defining %s\n",  t);
   
   SwigType *rettype = Copy(Getattr(n, "type"));
   SwigType *funcparams = SwigType_functionpointer_decompose(rettype);
@@ -555,13 +555,13 @@ String * R::createFunctionPointerHandler(SwigType *t, Node *n, int *numArgs) {
 
 
   if (debugMode) {
-    Printf(stderr, "Type: %s\n", t);
-    Printf(stderr, "Return type: %s\n", SwigType_base(t));
+    Printf(stdout, "Type: %s\n", t);
+    Printf(stdout, "Return type: %s\n", SwigType_base(t));
   }
   
   bool isVoidType = Strcmp(rettype, "void") == 0;
   if (debugMode)
-    Printf(stderr, "%s is void ? %s  (%s)\n", funName, isVoidType ? "yes" : "no", rettype);
+    Printf(stdout, "%s is void ? %s  (%s)\n", funName, isVoidType ? "yes" : "no", rettype);
   
   Wrapper *f = NewWrapper();
   
@@ -570,7 +570,7 @@ String * R::createFunctionPointerHandler(SwigType *t, Node *n, int *numArgs) {
   Parm *p = parms;
   for (i = 0; p; p = nextSibling(p), ++i) {
     String *arg = Getattr(p, "name");
-    String *lname = NewString("");
+    String *lname;
 
     if (!arg && Cmp(Getattr(p, "type"), "void")) {
       lname = NewStringf("s_arg%d", i+1);
@@ -608,7 +608,7 @@ String * R::createFunctionPointerHandler(SwigType *t, Node *n, int *numArgs) {
   if(numArgs) {
     *numArgs = nargs;
     if (debugMode)
-      Printf(stderr, "Setting number of parameters to %d\n", *numArgs);
+      Printf(stdout, "Setting number of parameters to %d\n", *numArgs);
   } 
   String *setExprElements = NewString("");
   
@@ -616,11 +616,16 @@ String * R::createFunctionPointerHandler(SwigType *t, Node *n, int *numArgs) {
   for(i = 0; p; i++) {
     SwigType *tt = Getattr(p, "type");
     SwigType *name = Getattr(p, "name");
-    //       String   *lname  = Getattr(p,"lname");
-    Printf(f->def,  "%s %s", SwigType_str(tt, 0), name);
     String *tm = Getattr(p, "tmap:out");
-    if(tm) {
+    Printf(f->def,  "%s %s", SwigType_str(tt, 0), name);
+     if(tm) {
       Replaceall(tm, "$1", name);
+      if (SwigType_isreference(tt)) {
+	String *tmp = NewString("");
+        Append(tmp, "*");
+	Append(tmp, name);
+	Replaceall(tm, tmp, name);
+      }
       Replaceall(tm, "$result", "r_tmp");
       replaceRClass(tm, Getattr(p,"type"));
       Replaceall(tm,"$owner", "R_SWIG_EXTERNAL");
@@ -675,12 +680,12 @@ String * R::createFunctionPointerHandler(SwigType *t, Node *n, int *numArgs) {
        XXX  Have to be a little more clever so that we can deal with struct A * - the * is getting lost.
        Is this still true? If so, will a SwigType_push() solve things?
     */
-    Parm *bbase = NewParm(rettype, "result", n);
-    String *returnTM = Swig_typemap_lookup("in", bbase, "result", f);
+    Parm *bbase = NewParmNode(rettype, n);
+    String *returnTM = Swig_typemap_lookup("in", bbase, Swig_cresult_name(), f);
     if(returnTM) {
       String *tm = returnTM;
       Replaceall(tm,"$input", "r_swig_cb_data->retValue");
-      Replaceall(tm,"$target", "result");
+      Replaceall(tm,"$target", Swig_cresult_name());
       replaceRClass(tm, rettype);
       Replaceall(tm,"$owner", "R_SWIG_EXTERNAL");
       Replaceall(tm,"$disown","0");
@@ -691,11 +696,14 @@ String * R::createFunctionPointerHandler(SwigType *t, Node *n, int *numArgs) {
   
   Printv(f->code, "R_SWIG_popCallbackFunctionData(1);\n", NIL);
   Printv(f->code, "\n", UnProtectWrapupCode, NIL);
-  
-  if(!isVoidType)
-    Printv(f->code,  "return result;\n", NIL);
+
+  if (SwigType_isreference(rettype)) {  
+    Printv(f->code,  "return *", Swig_cresult_name(), ";\n", NIL);
+  } else if(!isVoidType)
+    Printv(f->code,  "return ", Swig_cresult_name(), ";\n", NIL);
   
   Printv(f->code, "\n}\n", NIL);
+  Replaceall(f->code, "SWIG_exception_fail", "SWIG_exception_noreturn");
   
   /* To coerce correctly in S, we really want to have an extra/intermediate
      function that handles the scoerceout. 
@@ -716,6 +724,7 @@ String * R::createFunctionPointerHandler(SwigType *t, Node *n, int *numArgs) {
   Delete(rtype);
   Delete(rettype);
   Delete(funcparams);
+  DelWrapper(f);
   
   return funName;
 }
@@ -744,7 +753,7 @@ int R::cDeclaration(Node *n) {
   SwigType *t = Getattr(n, "type");
   SwigType *name = Getattr(n, "name");
   if (debugMode)
-    Printf(stderr, "cDeclaration (%s): %s\n", name, SwigType_lstr(t, 0));
+    Printf(stdout, "cDeclaration (%s): %s\n", name, SwigType_lstr(t, 0));
   return Language::cDeclaration(n);
 }
 #endif
@@ -827,7 +836,6 @@ int R::top(Node *n) {
   Delete(f_init);
 
   Delete(s_header);
-  Close(f_begin);
   Delete(f_runtime);
   Delete(f_begin);
 
@@ -846,7 +854,7 @@ int R::DumpCode(Node *n) {
   Printf(output_filename, "%s%s.R", SWIG_output_directory(), Rpackage);
   
 #ifdef R_SWIG_VERBOSE
-  Printf(stderr, "Writing S code to %s\n", output_filename);
+  Printf(stdout, "Writing S code to %s\n", output_filename);
 #endif
   
   File *scode = NewFile(output_filename, "w", SWIG_output_files());
@@ -861,8 +869,7 @@ int R::DumpCode(Node *n) {
   Printf(scode, "%s\n\n", s_classes);
   Printf(scode, "%s\n", sfile);
   
-  Close(scode);
-  //  Delete(scode);
+  Delete(scode);
   String *outfile = Getattr(n,"outfile");
   File *runtime = NewFile(outfile,"w", SWIG_output_files());
   if (!runtime) {
@@ -876,7 +883,6 @@ int R::DumpCode(Node *n) {
   Printf(runtime, "%s\n", f_wrapper);
   Printf(runtime, "%s\n", f_init);
 
-  Close(runtime);
   Delete(runtime);
 
   if(outputNamespaceInfo) {
@@ -897,7 +903,6 @@ int R::DumpCode(Node *n) {
     Printf(ns, "\nexportMethods(\n");
     writeListByLine(namespaceFunctions, ns, 1);
     Printf(ns, ")\n");
-    Close(ns);
     Delete(ns);
     Delete(s_namespace);
   }
@@ -935,15 +940,15 @@ int R::OutputClassMethodsTable(File *) {
   if (debugMode) {
     for(i = 0; i < n ; i++ ) {
       key = Getitem(keys, i);
-      Printf(stderr, "%d) %s\n", i, key);
+      Printf(stdout, "%d) %s\n", i, key);
       List *els = Getattr(tb, key);
       int nels = Len(els);
-      Printf(stderr, "\t");
+      Printf(stdout, "\t");
       for(int j = 0; j < nels; j+=2) {
-	Printf(stderr, "%s%s", Getitem(els, j), j < nels - 1 ? ", " : "");
-	Printf(stderr, "%s\n", Getitem(els, j+1));
+	Printf(stdout, "%s%s", Getitem(els, j), j < nels - 1 ? ", " : "");
+	Printf(stdout, "%s\n", Getitem(els, j+1));
       }
-      Printf(stderr, "\n");
+      Printf(stdout, "\n");
     }
   }
 
@@ -1097,12 +1102,11 @@ int R::OutputMemberReferenceMethod(String *className, int isSet,
     Printv(f->code, tab8, "f(x, value);\n", NIL);
     Printv(f->code, tab8, "x;\n", NIL); // make certain to return the S value.
   } else {
-    Printv(f->code, tab8, "formals(f)[[1]] = x;\n", NIL);
     if (varaccessor) {
       Printv(f->code, tab8,
-	     "if (is.na(match(name, vaccessors))) f else f(x);\n", NIL);
+	     "if (is.na(match(name, vaccessors))) function(...){f(x, ...)} else f(x);\n", NIL);
     } else {
-      Printv(f->code, tab8, "f;\n", NIL);
+      Printv(f->code, tab8, "function(...){f(x, ...)};\n", NIL);
     }
   }
   Printf(f->code, "}\n");
@@ -1289,11 +1293,9 @@ void R::addAccessor(String *memberName, Wrapper *wrapper, String *name,
   Append(l, tmp);
   // if we could put the wrapper in directly:       Append(l, Copy(sfun));
   if (debugMode)
-    Printf(stderr, "Adding accessor: %s (%s) => %s\n", memberName, name, tmp);
+    Printf(stdout, "Adding accessor: %s (%s) => %s\n", memberName, name, tmp);
 }
 
-#define Swig_overload_rank R_swig_overload_rank
-
 #define MAX_OVERLOAD 256
 
 struct Overloaded {
@@ -1304,7 +1306,7 @@ struct Overloaded {
 };
 
 
-static List * Swig_overload_rank(Node *n, 
+List * R::Swig_overload_rank(Node *n, 
 				 bool script_lang_wrapping) {
   Overloaded  nodes[MAX_OVERLOAD];
   int         nnodes = 0;
@@ -1362,7 +1364,9 @@ static List * Swig_overload_rank(Node *n,
 	  int   differ = 0;
 	  int   num_checked = 0;
 	  while (p1 && p2 && (num_checked < nodes[i].argc)) {
-	    //	  Printf(stdout,"p1 = '%s', p2 = '%s'\n", Getattr(p1,"type"), Getattr(p2,"type"));
+	    if (debugMode) {
+	      Printf(stdout,"p1 = '%s', p2 = '%s'\n", Getattr(p1,"type"), Getattr(p2,"type"));
+	    }
 	    if (checkAttribute(p1,"tmap:in:numinputs","0")) {
 	      p1 = Getattr(p1,"tmap:in:next");
 	      continue;
@@ -1373,6 +1377,9 @@ static List * Swig_overload_rank(Node *n,
 	    }
 	    String *t1 = Getattr(p1,"tmap:typecheck:precedence");
 	    String *t2 = Getattr(p2,"tmap:typecheck:precedence");
+	    if (debugMode) {
+	      Printf(stdout,"t1 = '%s', t2 = '%s'\n", t1, t2);
+	    }
 	    if ((!t1) && (!nodes[i].error)) {
 	      Swig_warning(WARN_TYPEMAP_TYPECHECK, Getfile(nodes[i].n), Getline(nodes[i].n),
 			   "Overloaded method %s not supported (no type checking rule for '%s').\n",
@@ -1553,6 +1560,9 @@ void R::dispatchFunction(Node *n) {
 
   Printf(f->def,
 	 "`%s` <- function(...) {", sfname);
+  if (debugMode) {
+    Swig_print_node(n);
+  }
   List *dispatch = Swig_overload_rank(n, true);
   int   nfunc = Len(dispatch);
   Printv(f->code, 
@@ -1587,30 +1597,59 @@ void R::dispatchFunction(Node *n) {
       }
       Printv(f->code, "if (", NIL);
       for (p =pi, j = 0 ; j < num_arguments ; j++) {
+	if (debugMode) {
+	  Swig_print_node(p);
+	}
 	String *tm = Swig_typemap_lookup("rtype", p, "", 0);
 	if(tm) {
 	  replaceRClass(tm, Getattr(p, "type"));
 	}
-	if (DohStrcmp(tm,"numeric")==0) {
-	Printf(f->code, "%sis.numeric(argv[[%d]])",
-	       j == 0 ? "" : " && ",
-	       j+1);
+
+	String *tmcheck = Swig_typemap_lookup("rtypecheck", p, "", 0);
+	if (tmcheck) {
+	  String *tmp = NewString("");
+	  Printf(tmp, "argv[[%d]]", j+1);
+	  Replaceall(tmcheck, "$arg", tmp);
+	  Printf(tmp, "argtype[%d]", j+1);
+	  Replaceall(tmcheck, "$argtype", tmp);
+	  if (tm) {
+	    Replaceall(tmcheck, "$rtype", tm);
+	  }
+	  if (debugMode) {
+	    Printf(stdout, "<rtypecheck>%s\n", tmcheck);
+	  }
+	  Printf(f->code, "%s(%s)",
+		 j == 0? "" : " && ",
+		 tmcheck);
+	  p = Getattr(p, "tmap:in:next");
+	  continue;
 	}
-	else if (DohStrcmp(tm,"integer")==0) {
-	Printf(f->code, "%s(is.integer(argv[[%d]]) || is.numeric(argv[[%d]]))",
-	       j == 0 ? "" : " && ",
-	       j+1, j+1);
+	if (tm) {
+	  if (Strcmp(tm,"numeric")==0) {
+	    Printf(f->code, "%sis.numeric(argv[[%d]])",
+		j == 0 ? "" : " && ",
+		j+1);
+	  }
+	  else if (Strcmp(tm,"integer")==0) {
+	    Printf(f->code, "%s(is.integer(argv[[%d]]) || is.numeric(argv[[%d]]))",
+		j == 0 ? "" : " && ",
+		j+1, j+1);
+	  }
+	  else if (Strcmp(tm,"character")==0) {
+	    Printf(f->code, "%sis.character(argv[[%d]])",
+		j == 0 ? "" : " && ",
+		j+1);
+	  }
+	  else {
+	    Printf(f->code, "%sextends(argtypes[%d], '%s')",
+		j == 0 ? "" : " && ",
+		j+1,
+		tm);
+	  }
 	}
-	else if (DohStrcmp(tm,"character")==0) {
-	Printf(f->code, "%sis.character(argv[[%d]])",
-	       j == 0 ? "" : " && ",
-	       j+1);
-	}
-	else {
-	Printf(f->code, "%sextends(argtypes[%d], '%s')",
-	       j == 0 ? "" : " && ",
-	       j+1,
-	       tm);
+	if (!SwigType_ispointer(Getattr(p, "type"))) {
+	  Printf(f->code, " && length(argv[[%d]]) == 1",
+	      j+1);
 	}
 	p = Getattr(p, "tmap:in:next");
       }
@@ -1641,7 +1680,7 @@ int R::functionWrapper(Node *n) {
   String *type = Getattr(n, "type"); 
   
   if (debugMode) {
-    Printf(stderr, 
+    Printf(stdout, 
 	   "<functionWrapper> %s %s %s\n", fname, iname, type);
   }
   String *overname = 0;
@@ -1660,7 +1699,7 @@ int R::functionWrapper(Node *n) {
   }
   
   if (debugMode) 
-    Printf(stderr, 
+    Printf(stdout, 
 	   "<functionWrapper> processing parameters\n");
   
   
@@ -1694,11 +1733,11 @@ int R::functionWrapper(Node *n) {
     }
   }
   if (debugMode) 
-    Printf(stderr, "<functionWrapper> unresolved_return_type %s\n",
+    Printf(stdout, "<functionWrapper> unresolved_return_type %s\n",
 	   unresolved_return_type);
   if(processing_member_access_function) {
     if (debugMode)
-      Printf(stderr, "<functionWrapper memberAccess> '%s' '%s' '%s' '%s'\n", 
+      Printf(stdout, "<functionWrapper memberAccess> '%s' '%s' '%s' '%s'\n", 
 	     fname, iname, member_name, class_name);
     
     if(opaqueClassDeclaration)
@@ -1757,7 +1796,7 @@ int R::functionWrapper(Node *n) {
 
   //    if(addCopyParam)
   if (debugMode)
-    Printf(stderr, "Adding a .copy argument to %s for %s = %s\n", 
+    Printf(stdout, "Adding a .copy argument to %s for %s = %s\n", 
 	   iname, type, addCopyParam ? "yes" : "no");
 
   Printv(f->def, "SWIGEXPORT SEXP\n", wname, " ( ", NIL);
@@ -1805,18 +1844,21 @@ int R::functionWrapper(Node *n) {
     String   *lname  = Getattr(p,"lname");
 
     // R keyword renaming
-    if (name && Swig_name_warning(p, 0, name, 0))
-      name = 0;
-
-    /* If we have a :: in the parameter name because we are accessing a static member of a class, say, then
-       we need to remove that prefix. */
-    while (Strstr(name, "::")) {
-      //XXX need to free.
-      name = NewStringf("%s", Strchr(name, ':') + 2);
-      if (debugMode)
-	Printf(stderr, "+++  parameter name with :: in it %s\n", name);
+    if (name) {
+      if (Swig_name_warning(p, 0, name, 0)) {
+	name = 0;
+      } else {
+	/* If we have a :: in the parameter name because we are accessing a static member of a class, say, then
+	   we need to remove that prefix. */
+	while (Strstr(name, "::")) {
+	  //XXX need to free.
+	  name = NewStringf("%s", Strchr(name, ':') + 2);
+	  if (debugMode)
+	    Printf(stdout, "+++  parameter name with :: in it %s\n", name);
+	}
+      }
     }
-    if (Len(name) == 0)
+    if (!name || Len(name) == 0)
       name = NewStringf("s_arg%d", i+1);
 
     name = replaceInitialDash(name);
@@ -1860,6 +1902,8 @@ int R::functionWrapper(Node *n) {
 	       "\n};\n",
 	       "if(is(", name, ", \"NativeSymbolInfo\")) {\n",
 	       name, " = ", name, "$address", ";\n}\n",
+	       "if(is(", name, ", \"ExternalReference\")) {\n",
+	       name, " = ", name, "@ref;\n}\n",
 	       "}; \n",
 	       NIL);
       } else {
@@ -1975,15 +2019,15 @@ int R::functionWrapper(Node *n) {
   String *actioncode = emit_action(n);
 
   /* Deal with the explicit return value. */
-  if ((tm = Swig_typemap_lookup_out("out", n, "result", f, actioncode))) { 
+  if ((tm = Swig_typemap_lookup_out("out", n, Swig_cresult_name(), f, actioncode))) { 
     SwigType *retType = Getattr(n, "type");
-    //Printf(stderr, "Return Value for %s, array? %s\n", retType, SwigType_isarray(retType) ? "yes" : "no");     
+    //Printf(stdout, "Return Value for %s, array? %s\n", retType, SwigType_isarray(retType) ? "yes" : "no");     
     /*      if(SwigType_isarray(retType)) {
 	    defineArrayAccessors(retType);
 	    } */
 
 
-    Replaceall(tm,"$1", "result");
+    Replaceall(tm,"$1", Swig_cresult_name());
     Replaceall(tm,"$result", "r_ans");
     replaceRClass(tm, retType);
 
@@ -2043,8 +2087,8 @@ int R::functionWrapper(Node *n) {
 
   /* Look to see if there is any newfree cleanup code */
   if (GetFlag(n, "feature:new")) {
-    if ((tm = Swig_typemap_lookup("newfree", n, "result", 0))) {
-      Replaceall(tm, "$source", "result");	/* deprecated */
+    if ((tm = Swig_typemap_lookup("newfree", n, Swig_cresult_name(), 0))) {
+      Replaceall(tm, "$source", Swig_cresult_name());	/* deprecated */
       Printf(f->code, "%s\n", tm);
     }
   }
@@ -2052,8 +2096,7 @@ int R::functionWrapper(Node *n) {
   Printv(f->code, UnProtectWrapupCode, NIL);
 
   /*If the user gave us something to convert the result in  */
-  if ((tm = Swig_typemap_lookup("scoerceout", n, 
-				    "result", sfun))) {
+  if ((tm = Swig_typemap_lookup("scoerceout", n, Swig_cresult_name(), sfun))) {
     Replaceall(tm,"$source","ans");
     Replaceall(tm,"$result","ans");
     replaceRClass(tm, Getattr(n, "type"));
@@ -2070,7 +2113,7 @@ int R::functionWrapper(Node *n) {
 	{ 
 	  String *finalizer = NewString(iname);
 	  Replace(finalizer, "new_", "", DOH_REPLACE_FIRST);
-	  Printf(sfun->code, "reg.finalizer(ans, delete_%s)\n", finalizer);
+	  Printf(sfun->code, "reg.finalizer(ans@ref, delete_%s)\n", finalizer);
 	}                                                                      
       Printf(sfun->code, "ans\n");
     }
@@ -2259,7 +2302,7 @@ int R::classDeclaration(Node *n) {
   if(Getattr(n, "unnamed") && Strcmp(Getattr(n, "storage"), "typedef") == 0 
      && Getattr(n, "tdname") && Strcmp(Getattr(n, "tdname"), name) == 0) {
     if (debugMode)
-      Printf(stderr, "Typedef in the class declaration for %s\n", name);
+      Printf(stdout, "Typedef in the class declaration for %s\n", name);
     //        typedefHandler(n);
   }
 
@@ -2306,7 +2349,7 @@ int R::classDeclaration(Node *n) {
       elName = Getattr(c, "name");
  
       String *elKind = Getattr(c, "kind");
-      if (Strcmp(elKind, "variable") != 0) {
+      if (!Equal(elKind, "variable")) {
 	c = nextSibling(c);
 	continue;
       }
@@ -2341,7 +2384,7 @@ int R::classDeclaration(Node *n) {
 	    
 	    
       // returns ""  tp = processType(elType, c, NULL);
-      //	    Printf(stderr, "<classDeclaration> elType %p\n", elType);
+      //	    Printf(stdout, "<classDeclaration> elType %p\n", elType);
       //	    tp = getRClassNameCopyStruct(Getattr(c, "type"), 1);
 #endif
       String *elNameT = replaceInitialDash(elName);
@@ -2393,7 +2436,7 @@ int R::generateCopyRoutines(Node *n) {
   String *mangledName = SwigType_manglestr(name);
 
   if (debugMode)
-    Printf(stderr, "generateCopyRoutines:  name = %s, %s\n", name, type);
+    Printf(stdout, "generateCopyRoutines:  name = %s, %s\n", name, type);
 
   Printf(copyToR->def, "CopyToR%s = function(value, obj = new(\"%s\"))\n{\n", 
 	 mangledName, name);
@@ -2408,8 +2451,7 @@ int R::generateCopyRoutines(Node *n) {
       continue;
     }
     String *elKind = Getattr(c, "kind");
-    if (Strcmp(elKind, "variable") != 0) {
-      Delete(elKind);
+    if (!Equal(elKind, "variable")) {
       continue;
     }
 
@@ -2476,7 +2518,7 @@ int R::typedefHandler(Node *n) {
   SwigType *tp = Getattr(n, "type");
   String *type = Getattr(n, "type");
   if (debugMode)
-    Printf(stderr, "<typedefHandler> %s\n", Getattr(n, "name"));
+    Printf(stdout, "<typedefHandler> %s\n", Getattr(n, "name"));
 
   processType(tp, n);
 
@@ -2485,7 +2527,7 @@ int R::typedefHandler(Node *n) {
     char *trueName = Char(type);
     trueName += 7;
     if (debugMode)
-      Printf(stderr, "<typedefHandler> Defining S class %s\n", trueName);
+      Printf(stdout, "<typedefHandler> Defining S class %s\n", trueName);
     Printf(s_classes, "setClass('_p%s', contains = 'ExternalReference')\n", 
 	   SwigType_manglestr(name));
   }
@@ -2507,13 +2549,13 @@ int R::membervariableHandler(Node *n) {
   processing_member_access_function = 1;
   member_name = Getattr(n,"sym:name");
   if (debugMode)
-    Printf(stderr, "<membervariableHandler> name = %s, sym:name = %s\n", 
+    Printf(stdout, "<membervariableHandler> name = %s, sym:name = %s\n", 
 	   Getattr(n, "name"), member_name);
 
   int status(Language::membervariableHandler(n));
 
   if(!opaqueClassDeclaration && debugMode)
-    Printf(stderr, "<membervariableHandler> %s %s\n", Getattr(n, "name"), Getattr(n, "type"));
+    Printf(stdout, "<membervariableHandler> %s %s\n", Getattr(n, "name"), Getattr(n, "type"));
 
   processing_member_access_function = 0;
   member_name = NULL;
@@ -2528,7 +2570,7 @@ int R::membervariableHandler(Node *n) {
 String * R::runtimeCode() {
   String *s = Swig_include_sys("rrun.swg");
   if (!s) {
-    Printf(stderr, "*** Unable to open 'rrun.swg'\n");
+    Printf(stdout, "*** Unable to open 'rrun.swg'\n");
     s = NewString("");
   }
   return s;
@@ -2615,6 +2657,13 @@ void R::main(int argc, char *argv[]) {
     if (cppcast) {
       Preprocessor_define((DOH *) "SWIG_CPLUSPLUS_CAST", 0);
     }
+
+    if (debugMode) {
+      Swig_typemap_search_debug_set();
+      Swig_typemap_used_debug_set();
+      Swig_typemap_register_debug_set();
+      Swig_file_debug_set();
+    }
     /// copyToR copyToC functions.
 
   }
@@ -2660,7 +2709,7 @@ String * R::processType(SwigType *t, Node *n, int *nargs) {
 
   SwigType *tmp = Getattr(n, "tdname");
   if (debugMode)
-    Printf(stderr, "processType %s (tdname = %s)\n", Getattr(n, "name"), tmp);
+    Printf(stdout, "processType %s (tdname = %s)\n", Getattr(n, "name"), tmp);
   
   SwigType *td = t;
   if (expandTypedef(t) &&
@@ -2677,7 +2726,7 @@ String * R::processType(SwigType *t, Node *n, int *nargs) {
     String *b = getRTypeName(t, &count);
     if(count && b && !Getattr(SClassDefs, b)) {
       if (debugMode)
-	Printf(stderr, "<processType> Defining class %s\n",  b);
+	Printf(stdout, "<processType> Defining class %s\n",  b);
 
       Printf(s_classes, "setClass('%s', contains = 'ExternalReference')\n", b);       
       Setattr(SClassDefs, b, b);
@@ -2691,7 +2740,7 @@ String * R::processType(SwigType *t, Node *n, int *nargs) {
 
   if(SwigType_isfunctionpointer(t)) {
     if (debugMode)
-      Printf(stderr, 
+      Printf(stdout, 
 	     "<processType> Defining pointer handler %s\n",  t);
        
     String *tmp = createFunctionPointerHandler(t, n, nargs);
