@@ -17,8 +17,6 @@
  * - Lines beginning with %# are stripped down to #... and passed through.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_cpp_c[] = "$Id: cpp.c 12215 2010-09-11 11:02:18Z wsfulton $";
-
 #include "swig.h"
 #include "preprocessor.h"
 #include <ctype.h>
@@ -33,6 +31,7 @@ static Hash *included_files = 0;
 static List *dependencies = 0;
 static Scanner *id_scan = 0;
 static int error_as_warning = 0;	/* Understand the cpp #error directive as a special #warning */
+static int expand_defined_operator = 0;
 static int macro_level = 0;
 static int macro_start_line = 0;
 static const String * macro_start_file = 0;
@@ -92,8 +91,6 @@ static String *cpp_include(const_String_or_char_ptr fn, int sysfile) {
     Setattr(included_files, file, file);
   }
   if (!s) {
-    /* XXX(bhy) may not need the seek */
-    /* Seek(fn, 0, SEEK_SET); */
     if (ignore_missing) {
       Swig_warning(WARN_PP_MISSING_FILE, Getfile(fn), Getline(fn), "Unable to find '%s'\n", fn);
     } else {
@@ -146,11 +143,15 @@ static String *kpp_dline = 0;
 static String *kpp_ddefine = 0;
 static String *kpp_dinclude = 0;
 static String *kpp_dimport = 0;
+static String *kpp_dbeginfile = 0;
 static String *kpp_dextern = 0;
 
 static String *kpp_LINE = 0;
 static String *kpp_FILE = 0;
 
+static String *kpp_hash_if = 0;
+static String *kpp_hash_elif = 0;
+
 void Preprocessor_init(void) {
   Hash *s;
 
@@ -179,6 +180,7 @@ void Preprocessor_init(void) {
 
   kpp_dinclude = NewString("%include");
   kpp_dimport = NewString("%import");
+  kpp_dbeginfile = NewString("%beginfile");
   kpp_dextern = NewString("%extern");
   kpp_ddefine = NewString("%define");
   kpp_dline = NewString("%line");
@@ -187,6 +189,9 @@ void Preprocessor_init(void) {
   kpp_LINE = NewString("__LINE__");
   kpp_FILE = NewString("__FILE__");
 
+  kpp_hash_if = NewString("#if");
+  kpp_hash_elif = NewString("#elif");
+
   cpp = NewHash();
   s = NewHash();
   Setattr(cpp, kpp_symbols, s);
@@ -224,13 +229,17 @@ void Preprocessor_delete(void) {
 
   Delete(kpp_dinclude);
   Delete(kpp_dimport);
+  Delete(kpp_dbeginfile);
   Delete(kpp_dextern);
   Delete(kpp_ddefine);
   Delete(kpp_dline);
 
-
   Delete(kpp_LINE);
   Delete(kpp_FILE);
+
+  Delete(kpp_hash_if);
+  Delete(kpp_hash_elif);
+
   Delete(cpp);
   Delete(included_files);
   Preprocessor_expr_delete();
@@ -572,7 +581,8 @@ static List *find_args(String *s, int ismacro, String *macro_name) {
   c = Getc(s);
   if (c != '(') {
     /* Not a macro, bail out now! */
-    Seek(s, pos, SEEK_SET);
+    assert(pos != -1);
+    (void)Seek(s, pos, SEEK_SET);
     Delete(args);
     return 0;
   }
@@ -633,7 +643,7 @@ unterm:
 }
 
 /* -----------------------------------------------------------------------------
- * DOH *get_filename(DOH *str)
+ * DOH *get_filename()
  *
  * Read a filename from str.   A filename can be enclosed in quotes, angle brackets,
  * or bare.
@@ -655,11 +665,30 @@ static String *get_filename(String *str, int *sysfile) {
     while (((c = Getc(str)) != EOF) && (c != '>'))
       Putc(c, fn);
   } else {
+    String *preprocessed_str;
     Putc(c, fn);
     while (((c = Getc(str)) != EOF) && (!isspace(c)))
       Putc(c, fn);
     if (isspace(c))
       Ungetc(c, str);
+    preprocessed_str = Preprocessor_replace(fn);
+    Seek(preprocessed_str, 0, SEEK_SET);
+    Delete(fn);
+
+    fn = NewStringEmpty();
+    copy_location(preprocessed_str, fn);
+    c = Getc(preprocessed_str);
+    if (c == '\"') {
+      while (((c = Getc(preprocessed_str)) != EOF) && (c != '\"'))
+	Putc(c, fn);
+    } else if (c == '<') {
+      *sysfile = 1;
+      while (((c = Getc(preprocessed_str)) != EOF) && (c != '>'))
+	Putc(c, fn);
+    } else {
+      fn = Copy(preprocessed_str);
+    }
+    Delete(preprocessed_str);
   }
   Swig_filename_unescape(fn);
   Swig_filename_correct(fn);
@@ -945,22 +974,6 @@ static String *expand_macro(String *name, List *args, String *line_file) {
   return e;
 }
 
-/* -----------------------------------------------------------------------------
- * evaluate_args()
- *
- * Evaluate the arguments of a macro 
- * ----------------------------------------------------------------------------- */
-
-List *evaluate_args(List *x) {
-  Iterator i;
-  List *nl = NewList();
-
-  for (i = First(x); i.item; i = Next(i)) {
-    Append(nl, Preprocessor_replace(i.item));
-  }
-  return nl;
-}
-
 /* -----------------------------------------------------------------------------
  * DOH *Preprocessor_replace(DOH *s)
  *
@@ -975,7 +988,6 @@ List *evaluate_args(List *x) {
 static DOH *Preprocessor_replace(DOH *s) {
   DOH *ns, *symbols, *m;
   int c, i, state = 0;
-
   String *id = NewStringEmpty();
 
   assert(cpp);
@@ -989,10 +1001,18 @@ static DOH *Preprocessor_replace(DOH *s) {
   while ((c = Getc(s)) != EOF) {
     switch (state) {
     case 0:
-      if (isidentifier(c) || (c == '%')) {
+      if (isidentifier(c)) {
 	Clear(id);
 	Putc(c, id);
-	state = 1;
+	state = 4;
+      } else if (c == '%') {
+	Clear(id);
+	Putc(c, id);
+	state = 2;
+      } else if (c == '#') {
+	Clear(id);
+	Putc(c, id);
+	state = 4;
       } else if (c == '\"') {
 	Putc(c, ns);
 	skip_tochar(s, '\"', ns);
@@ -1002,62 +1022,87 @@ static DOH *Preprocessor_replace(DOH *s) {
       } else if (c == '/') {
 	Putc(c, ns);
 	state = 10;
+      } else if (c == '\\') {
+	Putc(c, ns);
+	c = Getc(s);
+	if (c == '\n') {
+	  Putc(c, ns);
+	} else {
+	  Ungetc(c, s);
+	}
+      } else if (c == '\n') {
+	Putc(c, ns);
+	expand_defined_operator = 0;
       } else {
 	Putc(c, ns);
       }
       break;
-    case 1:			/* An identifier */
+    case 2:
+      /* Found '%#' */
+      if (c == '#') {
+	Putc(c, id);
+	state = 4;
+      } else {
+	Ungetc(c, s);
+	state = 4;
+      }
+      break;
+    case 4:			/* An identifier */
       if (isidchar(c)) {
 	Putc(c, id);
-	state = 1;
+	state = 4;
       } else {
 	/* We found the end of a valid identifier */
 	Ungetc(c, s);
-	/* See if this is the special "defined" macro */
-	if (Equal(kpp_defined, id)) {
-	  int lenargs = 0;
-	  DOH *args = 0;
-	  /* See whether or not a parenthesis has been used */
-	  skip_whitespace(s, 0);
-	  c = Getc(s);
-	  if (c == '(') {
-	    Ungetc(c, s);
-	    args = find_args(s, 0, kpp_defined);
-	  } else if (isidchar(c)) {
-	    DOH *arg = NewStringEmpty();
-	    args = NewList();
-	    Putc(c, arg);
-	    while (((c = Getc(s)) != EOF)) {
-	      if (!isidchar(c)) {
-		Ungetc(c, s);
-		break;
-	      }
+	/* See if this is the special "defined" operator */
+       	if (Equal(kpp_defined, id)) {
+	  if (expand_defined_operator) {
+	    int lenargs = 0;
+	    DOH *args = 0;
+	    /* See whether or not a parenthesis has been used */
+	    skip_whitespace(s, 0);
+	    c = Getc(s);
+	    if (c == '(') {
+	      Ungetc(c, s);
+	      args = find_args(s, 0, kpp_defined);
+	    } else if (isidchar(c)) {
+	      DOH *arg = NewStringEmpty();
+	      args = NewList();
 	      Putc(c, arg);
+	      while (((c = Getc(s)) != EOF)) {
+		if (!isidchar(c)) {
+		  Ungetc(c, s);
+		  break;
+		}
+		Putc(c, arg);
+	      }
+	      if (Len(arg))
+		Append(args, arg);
+	      Delete(arg);
+	    } else {
+	      Seek(s, -1, SEEK_CUR);
 	    }
-	    if (Len(arg))
-	      Append(args, arg);
-	    Delete(arg);
-	  } else {
-	    Seek(s, -1, SEEK_CUR);
-	  }
-	  lenargs = Len(args);
-	  if ((!args) || (!lenargs)) {
-	    /* This is not a defined() macro. */
-	    Append(ns, id);
-	    state = 0;
-	    break;
-	  }
-	  for (i = 0; i < lenargs; i++) {
-	    DOH *o = Getitem(args, i);
-	    if (!Getattr(symbols, o)) {
+	    lenargs = Len(args);
+	    if ((!args) || (!lenargs)) {
+	      /* This is not a defined() operator. */
+	      Append(ns, id);
+	      state = 0;
 	      break;
 	    }
+	    for (i = 0; i < lenargs; i++) {
+	      DOH *o = Getitem(args, i);
+	      if (!Getattr(symbols, o)) {
+		break;
+	      }
+	    }
+	    if (i < lenargs)
+	      Putc('0', ns);
+	    else
+	      Putc('1', ns);
+	    Delete(args);
+	  } else {
+	    Append(ns, id);
 	  }
-	  if (i < lenargs)
-	    Putc('0', ns);
-	  else
-	    Putc('1', ns);
-	  Delete(args);
 	  state = 0;
 	  break;
 	} else if (Equal(kpp_LINE, id)) {
@@ -1071,6 +1116,17 @@ static DOH *Preprocessor_replace(DOH *s) {
 	  Delete(fn);
 	  state = 0;
 	  break;
+	} else if (Equal(kpp_hash_if, id) || Equal(kpp_hash_elif, id)) {
+	  expand_defined_operator = 1;
+	  Append(ns, id);
+	  /*
+	} else if (Equal("%#if", id) || Equal("%#ifdef", id)) {
+	  Swig_warning(998, Getfile(s), Getline(s), "Found: %s preprocessor directive.\n", id);
+	  Append(ns, id);
+	} else if (Equal("#ifdef", id) || Equal("#ifndef", id)) {
+	  Swig_warning(998, Getfile(s), Getline(s), "The %s preprocessor directive does not work in macros, try #if instead.\n", id);
+	  Append(ns, id);
+	  */
 	} else if ((m = Getattr(symbols, id))) {
 	  /* See if the macro is defined in the preprocessor symbol table */
 	  DOH *args = 0;
@@ -1118,11 +1174,15 @@ static DOH *Preprocessor_replace(DOH *s) {
       Putc(c, ns);
       break;
     case 11:
+      /* in C++ comment */
       Putc(c, ns);
-      if (c == '\n')
+      if (c == '\n') {
+	expand_defined_operator = 0;
 	state = 0;
+      }
       break;
     case 12:
+      /* in C comment */
       Putc(c, ns);
       if (c == '*')
 	state = 13;
@@ -1141,8 +1201,8 @@ static DOH *Preprocessor_replace(DOH *s) {
   }
 
   /* Identifier at the end */
-  if (state == 1) {
-    /* See if this is the special "defined" macro */
+  if (state == 2 || state == 4) {
+    /* See if this is the special "defined" operator */
     if (Equal(kpp_defined, id)) {
       Swig_error(Getfile(s), Getline(s), "No arguments given to defined()\n");
     } else if (Equal(kpp_LINE, id)) {
@@ -1152,13 +1212,10 @@ static DOH *Preprocessor_replace(DOH *s) {
       Replaceall(fn, "\\", "\\\\");
       Printf(ns, "\"%s\"", fn);
       Delete(fn);
-    } else if ((m = Getattr(symbols, id))) {
+    } else if (Getattr(symbols, id)) {
       DOH *e;
       /* Yes.  There is a macro here */
       /* See if the macro expects arguments */
-      /*      if (Getattr(m,"args")) {
-         Swig_error(Getfile(id),Getline(id),"Macro arguments expected.\n");
-         } */
       e = expand_macro(id, 0, s);
       if (e)
 	Append(ns, e);
@@ -1269,6 +1326,7 @@ String *Preprocessor_parse(String *s) {
   int allow = 1;
   int level = 0;
   int dlevel = 0;
+  int filelevel = 0;
   int mask = 0;
   int start_level = 0;
   int cpp_lines = 0;
@@ -1511,9 +1569,14 @@ String *Preprocessor_parse(String *s) {
 	level++;
 	if (allow) {
 	  start_level = level;
-	  /* See if the identifier is in the hash table */
-	  if (!Getattr(symbols, value))
+	  if (Len(value) > 0) {
+	    /* See if the identifier is in the hash table */
+	    if (!Getattr(symbols, value))
+	      allow = 0;
+	  } else {
+	    Swig_error(Getfile(s), Getline(id), "Missing identifier for #ifdef.\n");
 	    allow = 0;
+	  }
 	  mask = 1;
 	}
       } else if (Equal(id, kpp_ifndef)) {
@@ -1521,9 +1584,14 @@ String *Preprocessor_parse(String *s) {
 	level++;
 	if (allow) {
 	  start_level = level;
-	  /* See if the identifier is in the hash table */
-	  if (Getattr(symbols, value))
+	  if (Len(value) > 0) {
+	    /* See if the identifier is in the hash table */
+	    if (Getattr(symbols, value))
+	      allow = 0;
+	  } else {
+	    Swig_error(Getfile(s), Getline(id), "Missing identifier for #ifndef.\n");
 	    allow = 0;
+	  }
 	  mask = 1;
 	}
       } else if (Equal(id, kpp_else)) {
@@ -1531,6 +1599,8 @@ String *Preprocessor_parse(String *s) {
 	  Swig_error(Getfile(s), Getline(id), "Misplaced #else.\n");
 	} else {
 	  cond_lines[level - 1] = Getline(id);
+	  if (Len(value) != 0)
+	    Swig_warning(WARN_PP_UNEXPECTED_TOKENS, Getfile(s), Getline(id), "Unexpected tokens after #else directive.\n");
 	  if (allow) {
 	    allow = 0;
 	    mask = 0;
@@ -1545,6 +1615,8 @@ String *Preprocessor_parse(String *s) {
 	  level = 0;
 	} else {
 	  if (level < start_level) {
+	    if (Len(value) != 0)
+	      Swig_warning(WARN_PP_UNEXPECTED_TOKENS, Getfile(s), Getline(id), "Unexpected tokens after #endif directive.\n");
 	    allow = 1;
 	    start_level--;
 	  }
@@ -1554,22 +1626,30 @@ String *Preprocessor_parse(String *s) {
 	level++;
 	if (allow) {
 	  int val;
-	  String *sval = Preprocessor_replace(value);
+	  String *sval;
+	  expand_defined_operator = 1;
+	  sval = Preprocessor_replace(value);
 	  start_level = level;
 	  Seek(sval, 0, SEEK_SET);
 	  /*      Printf(stdout,"Evaluating '%s'\n", sval); */
-	  val = Preprocessor_expr(sval, &e);
-	  if (e) {
-	    char *msg = Preprocessor_expr_error();
-	    Seek(value, 0, SEEK_SET);
-	    Swig_warning(WARN_PP_EVALUATION, Getfile(value), Getline(value), "Could not evaluate '%s'\n", value);
-	    if (msg)
-	      Swig_warning(WARN_PP_EVALUATION, Getfile(value), Getline(value), "Error: '%s'\n", msg);
-	    allow = 0;
-	  } else {
-	    if (val == 0)
+	  if (Len(sval) > 0) {
+	    val = Preprocessor_expr(sval, &e);
+	    if (e) {
+	      const char *msg = Preprocessor_expr_error();
+	      Seek(value, 0, SEEK_SET);
+	      Swig_warning(WARN_PP_EVALUATION, Getfile(value), Getline(value), "Could not evaluate expression '%s'\n", value);
+	      if (msg)
+		Swig_warning(WARN_PP_EVALUATION, Getfile(value), Getline(value), "Error: '%s'\n", msg);
 	      allow = 0;
+	    } else {
+	      if (val == 0)
+		allow = 0;
+	    }
+	  } else {
+	    Swig_error(Getfile(s), Getline(id), "Missing expression for #if.\n");
+	    allow = 0;
 	  }
+	  expand_defined_operator = 0;
 	  mask = 1;
 	}
       } else if (Equal(id, kpp_elif)) {
@@ -1582,27 +1662,35 @@ String *Preprocessor_parse(String *s) {
 	    mask = 0;
 	  } else if (level == start_level) {
 	    int val;
-	    String *sval = Preprocessor_replace(value);
+	    String *sval;
+	    expand_defined_operator = 1;
+	    sval = Preprocessor_replace(value);
 	    Seek(sval, 0, SEEK_SET);
-	    val = Preprocessor_expr(sval, &e);
-	    if (e) {
-	      char *msg = Preprocessor_expr_error();
-	      Seek(value, 0, SEEK_SET);
-	      Swig_warning(WARN_PP_EVALUATION, Getfile(value), Getline(value), "Could not evaluate '%s'\n", value);
-	      if (msg)
-		Swig_warning(WARN_PP_EVALUATION, Getfile(value), Getline(value), "Error: '%s'\n", msg);
-	      allow = 0;
-	    } else {
-	      if (val)
-		allow = 1 * mask;
-	      else
+	    if (Len(sval) > 0) {
+	      val = Preprocessor_expr(sval, &e);
+	      if (e) {
+		const char *msg = Preprocessor_expr_error();
+		Seek(value, 0, SEEK_SET);
+		Swig_warning(WARN_PP_EVALUATION, Getfile(value), Getline(value), "Could not evaluate expression '%s'\n", value);
+		if (msg)
+		  Swig_warning(WARN_PP_EVALUATION, Getfile(value), Getline(value), "Error: '%s'\n", msg);
 		allow = 0;
+	      } else {
+		if (val)
+		  allow = 1 * mask;
+		else
+		  allow = 0;
+	      }
+	    } else {
+	      Swig_error(Getfile(s), Getline(id), "Missing expression for #elif.\n");
+	      allow = 0;
 	    }
+	    expand_defined_operator = 0;
 	  }
 	}
       } else if (Equal(id, kpp_warning)) {
 	if (allow) {
-	  Swig_warning(WARN_PP_CPP_WARNING, Getfile(s), Getline(id), "CPP #warning, %s\n", value);
+	  Swig_warning(WARN_PP_CPP_WARNING, Getfile(s), Getline(id), "CPP #warning, \"%s\".\n", value);
 	}
       } else if (Equal(id, kpp_error)) {
 	if (allow) {
@@ -1616,10 +1704,10 @@ String *Preprocessor_parse(String *s) {
       } else if (Equal(id, kpp_include)) {
 	if (((include_all) || (import_all)) && (allow)) {
 	  String *s1, *s2, *fn;
-	  char *dirname;
+	  String *dirname;
 	  int sysfile = 0;
 	  if (include_all && import_all) {
-	    Swig_warning(WARN_PP_INCLUDEALL_IMPORTALL, Getfile(s), Getline(id), "Both includeall and importall are defined: using includeall\n");
+	    Swig_warning(WARN_PP_INCLUDEALL_IMPORTALL, Getfile(s), Getline(id), "Both includeall and importall are defined: using includeall.\n");
 	    import_all = 0;
 	  }
 	  Seek(value, 0, SEEK_SET);
@@ -1627,23 +1715,26 @@ String *Preprocessor_parse(String *s) {
 	  s1 = cpp_include(fn, sysfile);
 	  if (s1) {
 	    if (include_all)
-	      Printf(ns, "%%includefile \"%s\" [\n", Swig_filename_escape(Swig_last_file()));
+	      Printf(ns, "%%includefile \"%s\" %%beginfile\n", Swig_filename_escape(Swig_last_file()));
 	    else if (import_all) {
-	      Printf(ns, "%%importfile \"%s\" [\n", Swig_filename_escape(Swig_last_file()));
+	      Printf(ns, "%%importfile \"%s\" %%beginfile\n", Swig_filename_escape(Swig_last_file()));
 	      push_imported();
 	    }
 
 	    /* See if the filename has a directory component */
 	    dirname = Swig_file_dirname(Swig_last_file());
-	    if (sysfile || !strlen(dirname))
+	    if (sysfile || !Len(dirname)) {
+	      Delete(dirname);
 	      dirname = 0;
+	    }
 	    if (dirname) {
-	      dirname[strlen(dirname) - 1] = 0;	/* Kill trailing directory delimiter */
+	      int len = Len(dirname);
+	      Delslice(dirname, len - 1, len); /* Kill trailing directory delimiter */
 	      Swig_push_directory(dirname);
 	    }
 	    s2 = Preprocessor_parse(s1);
 	    addline(ns, s2, allow);
-	    Append(ns, "]");
+	    Append(ns, "%endoffile");
 	    if (dirname) {
 	      Swig_pop_directory();
 	    }
@@ -1651,6 +1742,7 @@ String *Preprocessor_parse(String *s) {
 	      pop_imported();
 	    }
 	    Delete(s2);
+	    Delete(dirname);
 	    Delete(s1);
 	  }
 	  Delete(fn);
@@ -1749,11 +1841,11 @@ String *Preprocessor_parse(String *s) {
 	Ungetc(c, s);
 	/* Look for common SWIG directives  */
 	if (Equal(decl, kpp_dinclude) || Equal(decl, kpp_dimport) || Equal(decl, kpp_dextern)) {
-	  /* Got some kind of file inclusion directive  */
+	  /* Got some kind of file inclusion directive, eg: %import(option1="value1") "filename" */
 	  if (allow) {
 	    DOH *s1, *s2, *fn, *opt;
-	    String *options_whitespace = NewString("");
-	    String *filename_whitespace = NewString("");
+	    String *options_whitespace = NewStringEmpty();
+	    String *filename_whitespace = NewStringEmpty();
 	    int sysfile = 0;
 
 	    if (Equal(decl, kpp_dextern)) {
@@ -1763,22 +1855,26 @@ String *Preprocessor_parse(String *s) {
 	    }
 	    skip_whitespace(s, options_whitespace);
 	    opt = get_options(s);
+
 	    skip_whitespace(s, filename_whitespace);
 	    fn = get_filename(s, &sysfile);
 	    s1 = cpp_include(fn, sysfile);
 	    if (s1) {
-	      char *dirname;
+	      String *dirname;
 	      copy_location(s, chunk);
 	      add_chunk(ns, chunk, allow);
-	      Printf(ns, "%sfile%s%s%s\"%s\" [\n", decl, options_whitespace, opt, filename_whitespace, Swig_filename_escape(Swig_last_file()));
+	      Printf(ns, "%sfile%s%s%s\"%s\" %%beginfile\n", decl, options_whitespace, opt, filename_whitespace, Swig_filename_escape(Swig_last_file()));
 	      if (Equal(decl, kpp_dimport)) {
 		push_imported();
 	      }
 	      dirname = Swig_file_dirname(Swig_last_file());
-	      if (sysfile || !strlen(dirname))
+	      if (sysfile || !Len(dirname)) {
+		Delete(dirname);
 		dirname = 0;
+	      }
 	      if (dirname) {
-		dirname[strlen(dirname) - 1] = 0;	/* Kill trailing directory delimiter */
+		int len = Len(dirname);
+		Delslice(dirname, len - 1, len); /* Kill trailing directory delimiter */
 		Swig_push_directory(dirname);
 	      }
 	      s2 = Preprocessor_parse(s1);
@@ -1789,8 +1885,9 @@ String *Preprocessor_parse(String *s) {
 		pop_imported();
 	      }
 	      addline(ns, s2, allow);
-	      Append(ns, "]");
+	      Append(ns, "%endoffile");
 	      Delete(s2);
+	      Delete(dirname);
 	      Delete(s1);
 	    }
 	    Delete(fn);
@@ -1798,6 +1895,14 @@ String *Preprocessor_parse(String *s) {
 	    Delete(options_whitespace);
 	  }
 	  state = 1;
+	} else if (Equal(decl, kpp_dbeginfile)) {
+          /* Got an internal directive marking the beginning of an included file: %beginfile ... %endoffile */
+          filelevel++;
+          start_line = Getline(s);
+          copy_location(s, chunk);
+          add_chunk(ns, chunk, allow);
+          Append(chunk, decl);
+          state = 120;
 	} else if (Equal(decl, kpp_dline)) {
 	  /* Got a line directive  */
 	  state = 1;
@@ -1818,6 +1923,40 @@ String *Preprocessor_parse(String *s) {
       }
       break;
 
+      /* Searching for the end of a %beginfile block */
+    case 120:
+      Putc(c, chunk);
+      if (c == '%') {
+        const char *bf = "beginfile";
+        const char *ef = "endoffile";
+        char statement[10];
+        int i = 0;
+        for (i = 0; i < 9;) {
+          c = Getc(s);
+          Putc(c, chunk);
+          statement[i++] = (char)c;
+	  if (strncmp(statement, bf, i) && strncmp(statement, ef, i))
+	    break;
+	}
+	c = Getc(s);
+	Ungetc(c, s);
+        if ((i == 9) && (isspace(c))) {
+	  if (strncmp(statement, bf, i) == 0) {
+	    ++filelevel;
+	  } else if (strncmp(statement, ef, i) == 0) {
+            --filelevel;
+            if (!filelevel) {
+              /* Reached end of included file */
+              addline(ns, chunk, allow);
+              Clear(chunk);
+              copy_location(s, chunk);
+              state = 1;
+            }
+          }
+        }
+      }
+      break;
+
       /* Searching for the end of a %define statement  */
     case 150:
       Putc(c, value);
@@ -1868,6 +2007,9 @@ String *Preprocessor_parse(String *s) {
     Swig_error(Getfile(s), -1, "Missing #endif for conditional starting on line %d\n", cond_lines[level - 1]);
     level--;
   }
+  if (state == 120) {
+    Swig_error(Getfile(s), -1, "Missing %%endoffile for file inclusion block starting on line %d\n", start_line);
+  }
   if (state == 150) {
     Seek(value, 0, SEEK_SET);
     Swig_error(Getfile(s), -1, "Missing %%enddef for macro starting on line %d\n", Getline(value));
@@ -1878,6 +2020,7 @@ String *Preprocessor_parse(String *s) {
   if ((state >= 30) && (state < 40)) {
     Swig_error(Getfile(s), -1, "Unterminated comment starting on line %d\n", start_line);
   }
+
   copy_location(s, chunk);
   add_chunk(ns, chunk, allow);
 
