@@ -16,10 +16,9 @@
  * wrapper code and to generate cleaner wrapper files. 
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_fragment_c[] = "$Id: fragment.c 11876 2010-02-27 23:53:33Z wsfulton $";
-
 #include "swig.h"
 #include "swigwarn.h"
+#include "cparse.h"
 
 static Hash *fragments = 0;
 static Hash *looking_fragments = 0;
@@ -60,6 +59,10 @@ void Swig_fragment_register(Node *fragment) {
       if (kwargs) {
 	Setmeta(ccode, "kwargs", kwargs);
       }
+      Setfile(ccode, Getfile(fragment));
+      Setline(ccode, Getline(fragment));
+      /* Replace $descriptor() macros */
+      Swig_cparse_replace_descriptor(ccode);
       Setattr(fragments, name, ccode);
       if (debug)
 	Printf(stdout, "registering fragment %s %s\n", name, section);
@@ -142,7 +145,7 @@ void Swig_fragment_emit(Node *n) {
       if (section) {
 	File *f = Swig_filebyname(section);
 	if (!f) {
-	  Swig_error(Getfile(code), Getline(code), "Bad section '%s' for code fragment '%s'\n", section, name);
+	  Swig_error(Getfile(code), Getline(code), "Bad section '%s' in %%fragment declaration for code fragment '%s'\n", section, name);
 	} else {
 	  if (debug)
 	    Printf(stdout, "emitting subfragment %s %s\n", name, section);
