@@ -13,8 +13,6 @@
  * are provided.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_include_c[] = "$Id: include.c 12930 2012-03-18 04:58:08Z drjoe $";
-
 #include "swig.h"
 
 static List   *directories = 0;	        /* List of include directories */
@@ -165,7 +163,8 @@ static FILE *Swig_open_file(const_String_or_char_ptr name, int sysfile, int use_
   String *filename;
   List *spath = 0;
   char *cname;
-  int i, ilen;
+  int i, ilen, nbytes;
+  char bom[3];
 
   if (!directories)
     directories = NewList();
@@ -193,6 +192,14 @@ static FILE *Swig_open_file(const_String_or_char_ptr name, int sysfile, int use_
   if (f) {
     Delete(lastpath);
     lastpath = filename;
+
+    /* Skip the UTF-8 BOM if it's present */
+    nbytes = fread(bom, 1, 3, f);
+    if (nbytes == 3 && bom[0] == (char)0xEF && bom[1] == (char)0xBB && bom[2] == (char)0xBF) {
+      /* skip */
+    } else {
+      fseek(f, 0, SEEK_SET);
+    }
   }
   return f;
 }
@@ -317,58 +324,41 @@ File *Swig_filebyname(const_String_or_char_ptr filename) {
 }
 
 /* -----------------------------------------------------------------------------
- * Swig_file_suffix()
+ * Swig_file_extension()
  *
- * Returns the suffix of a file
+ * Returns the extension of a file
  * ----------------------------------------------------------------------------- */
 
-char *Swig_file_suffix(const_String_or_char_ptr filename) {
-  char *d;
-  char *c = Char(filename);
-  int len = Len(filename);
-  if (strlen(c)) {
-    d = c + len - 1;
-    while (d != c) {
-      if (*d == '.')
-	return d;
-      d--;
-    }
-    return c + len;
-  }
-  return c;
+String *Swig_file_extension(const_String_or_char_ptr filename) {
+  String *name = Swig_file_filename(filename);
+  const char *c = strrchr(Char(name), '.');
+  String *extension = c ? NewString(c) : NewString("");
+  Delete(name);
+  return extension;
 }
 
 /* -----------------------------------------------------------------------------
  * Swig_file_basename()
  *
- * Returns the filename with no suffix attached.
+ * Returns the filename with the extension removed.
  * ----------------------------------------------------------------------------- */
 
-char *Swig_file_basename(const_String_or_char_ptr filename) {
-  static char tmp[1024];
-  char *c;
-  strcpy(tmp, Char(filename));
-  c = Swig_file_suffix(tmp);
-  *c = 0;
-  return tmp;
+String *Swig_file_basename(const_String_or_char_ptr filename) {
+  String *extension = Swig_file_extension(filename);
+  String *basename = NewStringWithSize(filename, Len(filename) - Len(extension));
+  Delete(extension);
+  return basename;
 }
 
 /* -----------------------------------------------------------------------------
  * Swig_file_filename()
  *
- * Return the file with any leading path stripped off
+ * Return the file name with any leading path stripped off
  * ----------------------------------------------------------------------------- */
-char *Swig_file_filename(const_String_or_char_ptr filename) {
-  static char tmp[1024];
+String *Swig_file_filename(const_String_or_char_ptr filename) {
   const char *delim = SWIG_FILE_DELIMITER;
-  char *c;
-
-  strcpy(tmp, Char(filename));
-  c = strrchr(tmp, *delim);
-  if (c)
-    return c + 1;
-  else
-    return tmp;
+  const char *c = strrchr(Char(filename), *delim);
+  return c ? NewString(c + 1) : NewString(filename);
 }
 
 /* -----------------------------------------------------------------------------
@@ -376,19 +366,10 @@ char *Swig_file_filename(const_String_or_char_ptr filename) {
  *
  * Return the name of the directory associated with a file
  * ----------------------------------------------------------------------------- */
-char *Swig_file_dirname(const_String_or_char_ptr filename) {
-  static char tmp[1024];
+String *Swig_file_dirname(const_String_or_char_ptr filename) {
   const char *delim = SWIG_FILE_DELIMITER;
-  char *c;
-  strcpy(tmp, Char(filename));
-  if (!strstr(tmp, delim)) {
-    return "";
-  }
-  c = tmp + strlen(tmp) - 1;
-  while (*c != *delim)
-    c--;
-  *(++c) = 0;
-  return tmp;
+  const char *c = strrchr(Char(filename), *delim);
+  return c ? NewStringWithSize(filename, c - Char(filename) + 1) : NewString("");
 }
 
 /*
